/*
 * Copyright (C) 2002, 2003, 2004 Emmanuel Saracco <esaracco@noos.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "utils.h"
#include "application.h"
#include "check.h"
#include "project.h"
#include "html_parser.h"

#define UC_HTML_PARSER_WHILE_CONDITION \
        ( \
                  (uc_check_cancel_get_value () == FALSE) && \
                  (uc_check_ignore_item_get_value () == FALSE) \
        )


static gchar *href_base = NULL;

static gboolean uc_html_parser_authorized_tag_name (const gchar * name);
static gchar *uc_html_parser_add_base (const gchar * base, const gchar * url);
static GList *uc_html_parser_parse (GList * tags, htmlNodePtr cur);
static UCLinkType uc_html_parser_get_tag_type (htmlNodePtr cur);
static gchar *uc_html_parser_get_tag_label (htmlNodePtr cur);
static gchar *uc_html_parser_get_tag_value (htmlNodePtr cur, UCLinkType type);
static GList *uc_html_parser_parse (GList * tags, htmlNodePtr cur);

/**
 * uc_html_parser_authorized_tag_name:
 * @name: Tag to check.
 * 
 * Check if a given tag must be proceed or not.
 * 
 * Returns: TRUE if the given name is usefull for us.
 */
static gboolean
uc_html_parser_authorized_tag_name (const gchar * name)
{
  return (name != NULL &&
	  (g_ascii_strcasecmp (name, "IMG") == 0 ||
	   g_ascii_strcasecmp (name, "A") == 0 ||
	   g_ascii_strcasecmp (name, "BASE") == 0 ||
	   g_ascii_strcasecmp (name, "META") == 0 ||
	   g_ascii_strcasecmp (name, "LINK") == 0 ||
	   g_ascii_strcasecmp (name, "FRAME") == 0 ||
	   g_ascii_strcasecmp (name, "IFRAME") == 0));
}

/**
 * uc_html_parser_get_tags:
 * @buffer_orig: Buffer containing HTML code to proceed.
 * @base: HTML base tag.
 * 
 * Parse the given buffer to extract HTML tags.
 *
 * Returns: A #GList of #UCHTMLTag.
 */
GList *
uc_html_parser_get_tags (gchar * buffer_orig, gchar ** base)
{
  GList *tags = NULL;
  gchar *buffer = NULL;
  htmlParserCtxtPtr ctxt = NULL;
  htmlDocPtr doc = NULL;

  uc_application_display_state_message (UC_CHECK_MESSAGE_LABEL_THIRD,
					_("Parsing HTML code..."));
  UC_UPDATE_UI;

  buffer = uc_utils_replace (buffer_orig, "&", "&amp;");
  ctxt = htmlCreateMemoryParserCtxt (buffer, strlen (buffer));
  htmlCtxtUseOptions (ctxt, HTML_PARSE_NOWARNING | HTML_PARSE_NOERROR);
  htmlParseDocument (ctxt);

  doc = ctxt->myDoc;
  if (doc == NULL)
    g_warning ("Bad HTML document");
  else
    tags = uc_html_parser_parse (tags, doc->children);

  g_free (buffer), buffer = NULL;
  htmlFreeParserCtxt (ctxt);

  if (href_base != NULL)
    {
      *base = g_strdup (href_base);

      g_free (href_base), href_base = NULL;
    }

  return tags;
}

GList *
uc_css_parser_get_tags (gchar * buffer_orig)
{
  GList *tags = NULL;
  gchar *ext = NULL;
  gchar *tmp = NULL;
  gchar *buffer = buffer_orig;

  uc_application_display_state_message (UC_CHECK_MESSAGE_LABEL_THIRD,
					_("Parsing CSS code..."));
  UC_UPDATE_UI;

  while (*buffer != '\0' && (buffer = strpbrk (buffer, "@uU")) != NULL)
    {
      UCHTMLTag *tag = NULL;
      gchar *url = NULL;
      gchar *b = NULL;
      gchar *e = NULL;

      if (uc_utils_memcasecmp (buffer, "@import") == TRUE)
	{
	  b = (buffer += 8);

	  if ((b = uc_utils_strpbrk_or_eos (b, "('\"")) != NULL);
	  {
	    do
	      b++;
	    while (*b == ' ' || *b == '\'' || *b == '"');

	    if ((e = uc_utils_strpbrk_or_eos (b, "'\";) ")) != NULL)
	      {
		tmp = uc_utils_strdup_delim (b, e);
		url = uc_utils_clean_tag_link_value (tmp);
		g_free (tmp), tmp = NULL;

		/* file must have a extension */
		if ((ext = strrchr (url, '.')) != NULL)
		  {
		    tag = uc_html_parser_node_new ();
		    tag->type = LINK_TYPE_CSS;
		  }
		else
		  g_free (url), url = NULL;
	      }
	  }
	}
      else if (uc_utils_memcasecmp (buffer, "url") == TRUE)
	{
	  b = (buffer += 4);

	  /* we do not want "url*s*" tag */
	  if (*(b - 1) != 's' && (e = strchr (b, ')')) != NULL)
	    {
	      tmp = uc_utils_strdup_delim (b, e);
	      url = uc_utils_clean_tag_link_value (tmp);
	      g_free (tmp), tmp = NULL;

	      /* file must have a extension */
	      if ((ext = strrchr (url, '.')) != NULL)
		{
		  tag = uc_html_parser_node_new ();
		  tag->type = (strncasecmp (".css", ext, 3) == 0) ?
		    LINK_TYPE_CSS :
		    (strncasecmp (url, UC_PROTOCOL_FILE,
				  strlen (UC_PROTOCOL_FILE) - 1) == 0) ?
		    LINK_TYPE_FILE_IMAGE : LINK_TYPE_IMAGE;
		}
	      else
		g_free (url), url = NULL;
	    }
	}

      if (tag != NULL)
	{
	  tag->label = g_strdup ("");
	  tag->value = url;
	  uc_utils_strstrip (tag->value);
	  tags = g_list_append (tags, tag);
	}

      if (*buffer != '\0')
	buffer++;
    }

  return tags;
}

/**
 * uc_html_parser_get_tag_type:
 * @cur: XML cursor pointer.
 *
 * Proceed a HTML tag to see its type.
 * 
 * Returns: #UCLinkType of the HTML tag.
 */
static UCLinkType
uc_html_parser_get_tag_type (htmlNodePtr cur)
{
  xmlAttrPtr attr = NULL;
  UCLinkType ret = LINK_TYPE_NONE;

  if (g_ascii_strcasecmp ((gchar *) cur->name, "BASE") == 0)
    ret = LINK_TYPE_BASE_HREF;
  else if (g_ascii_strcasecmp ((gchar *) cur->name, "META") == 0)
    ret = LINK_TYPE_META;
  else if (g_ascii_strcasecmp ((gchar *) cur->name, "IMG") == 0)
    ret = LINK_TYPE_IMAGE;
  else if (g_ascii_strcasecmp ((gchar *) cur->name, "LINK") == 0)
    {
      attr = cur->properties;
      while (attr != NULL && ret == LINK_TYPE_NONE)
	{
	  if (attr->name != NULL &&
	      g_ascii_strcasecmp ((gchar *) attr->name, "REL") == 0 &&
	      attr->children != NULL && attr->children->content != NULL &&
	      uc_utils_memcasecmp ((gchar *) attr->children->content,
				   "stylesheet") == TRUE)
	    ret = LINK_TYPE_CSS;

	  attr = attr->next;
	}
    }
  else if (g_ascii_strcasecmp ((gchar *) cur->name, "A") == 0)
    {
      attr = cur->properties;
      while (attr != NULL && ret == LINK_TYPE_NONE)
	{
	  ret = LINK_TYPE_HREF;

	  if (attr->name != NULL &&
	      g_ascii_strcasecmp ((gchar *) attr->name, "HREF") == 0 &&
	      attr->children != NULL && attr->children->content != NULL)
	    {
	      if (uc_utils_memcasecmp ((gchar *) attr->children->content,
				       "MAILTO:") == TRUE)
		ret = LINK_TYPE_EMAIL;
	      else
		if (uc_utils_memcasecmp ((gchar *) attr->children->content,
					 "FILE:") == TRUE)
		ret = LINK_TYPE_FILE_HREF;
	    }

	  attr = attr->next;
	}
    }
  else if (g_ascii_strcasecmp ((gchar *) cur->name, "FRAME") == 0 ||
	   g_ascii_strcasecmp ((gchar *) cur->name, "IFRAME") == 0)
    ret = LINK_TYPE_FRAME;

  return ret;
}

/**
 * uc_html_parser_get_tag_imbricated_label:
 * @cur: XML cursor pointer.
 * @ret: A pointer that will point on the tag label. 
 *
 * This function is called when tag label contain HTML tags.
 * It fill the @ret argument with the label of the given tag, without HTML
 * tags. @ret will point on a new allocated string.
 */
static void
uc_html_parser_get_tag_imbricated_label (htmlNodePtr cur, gchar ** ret)
{
  while (cur != NULL)
    {
      if (cur->name != NULL &&
	  g_ascii_strcasecmp ((gchar *) cur->name, "text") == 0 &&
	  cur->content != NULL)
	{
	  if (*ret == NULL)
	    *ret = g_strdup ((gchar *) cur->content);
	  else
	    {
	      gchar *sav = NULL;

	      sav = g_strdup_printf ("%s %s", *ret, cur->content);
	      g_free (*ret), *ret = NULL;
	      *ret = sav;
	    }
	}

      if (cur->children != NULL)
	uc_html_parser_get_tag_imbricated_label (cur->children, &(*ret));

      cur = cur->next;
    }
}

/**
 * uc_html_parser_get_tag_label:
 * @cur: XML cursor pointer.
 *
 * Proceed a given HTML tag to extract its label. If label contain
 * other HTML tags, those are deleted.
 * 
 * See: uc_html_parser_get_tag_imbricated_label(), 
 *      uc_html_parser_get_tag_value()
 *      
 * Returns: Label of the tag.
 */
static gchar *
uc_html_parser_get_tag_label (htmlNodePtr cur)
{
  gchar *ret = NULL;
  xmlAttrPtr attr = NULL;

  if (cur->children != NULL)
    {
      gchar *tmp = NULL;

      uc_html_parser_get_tag_imbricated_label (cur->children, &tmp);

      if (tmp != NULL)
	{
	  ret = uc_utils_to_utf8 (tmp);
	  g_free (tmp), tmp = NULL;
	}
    }
  else
    {
      attr = cur->properties;
      while (attr != NULL && ret == NULL)
	{
	  if (attr->name != NULL && attr->children != NULL
	      && (g_ascii_strcasecmp ((gchar *) attr->name, "TITLE") == 0
		  || g_ascii_strcasecmp ((gchar *) attr->name, "ALT") == 0
		  || g_ascii_strcasecmp ((gchar *) attr->name, "NAME") == 0
		  || g_ascii_strcasecmp ((gchar *) attr->name,
					 "HTTP-EQUIV") == 0))
	    ret = uc_utils_to_utf8 ((gchar *) attr->children->content);

	  attr = attr->next;
	}
    }

  if (ret == NULL)
    ret = g_strdup ("");

  return ret;
}

/**
 * uc_html_parser_get_tag_value:
 * @cur: XML cursor pointer.
 * @type: Tag type.
 *
 * Proceed a given tag to extract its value.
 * 
 * See: uc_html_parser_get_tag_label()
 * 
 * Returns: The value of the tag.
 */
static gchar *
uc_html_parser_get_tag_value (htmlNodePtr cur, UCLinkType type)
{
  xmlAttrPtr attr = NULL;
  gchar *ret = NULL;

  switch (type)
    {
    case LINK_TYPE_IMAGE:
    case LINK_TYPE_FILE_IMAGE:
    case LINK_TYPE_FRAME:
    case LINK_TYPE_EMAIL:
    case LINK_TYPE_BASE_HREF:
    case LINK_TYPE_FILE_HREF:
    case LINK_TYPE_HREF:
    case LINK_TYPE_CSS:
      attr = cur->properties;
      while (attr != NULL && ret == NULL)
	{
	  if (attr->name != NULL &&
	      (g_ascii_strcasecmp ((gchar *) attr->name, "HREF") == 0 ||
	       g_ascii_strcasecmp ((gchar *) attr->name, "SRC") == 0)
	      && attr->children != NULL && attr->children->content != NULL)
	    ret = uc_utils_to_utf8 ((gchar *) attr->children->content);

	  attr = attr->next;
	}
      break;

    case LINK_TYPE_META:
      attr = cur->properties;
      while (attr != NULL && ret == NULL)
	{
	  if (attr->name != NULL && g_ascii_strcasecmp ((gchar *) attr->name,
							"CONTENT") == 0
	      && attr->children != NULL && attr->children->content != NULL)
	    ret = uc_utils_to_utf8 ((gchar *) attr->children->content);

	  attr = attr->next;
	}
      break;

    default:
      ;
    }

  if (ret == NULL)
    ret = g_strdup ("");

  return ret;
}

/**
 * uc_html_parser_parse:
 * @tags: #GList of #UCHTMLTag items.
 * @cur: XML cursor pointer.
 * 
 * Parse a given XML structure to extract HTML tags.
 * 
 * Returns: A #GList of #UCHTMLTag.
 */
static GList *
uc_html_parser_parse (GList * tags, htmlNodePtr cur)
{
  gchar *tmp = NULL;

  while (cur != NULL && UC_HTML_PARSER_WHILE_CONDITION)
    {
      if (cur->type == XML_ELEMENT_NODE && cur->name != NULL &&
	  uc_html_parser_authorized_tag_name ((gchar *) cur->name))
	{
	  UCHTMLTag *tag = NULL;

	  tag = uc_html_parser_node_new ();
	  tag->type = uc_html_parser_get_tag_type (cur);
	  tag->label = uc_html_parser_get_tag_label (cur);
	  uc_utils_strstrip (tag->label);
	  tag->value = uc_html_parser_get_tag_value (cur, tag->type);
	  uc_utils_strstrip (tag->value);

	  switch (tag->type)
	    {
	    case LINK_TYPE_META:
	    case LINK_TYPE_EMAIL:
	    case LINK_TYPE_FILE_HREF:
	      break;

	    case LINK_TYPE_BASE_HREF:
	      if (href_base == NULL)
		{
		  href_base = g_strdup (tag->value);
		  if (strlen (href_base) >= 2
		      && (href_base)[strlen (href_base) - 1] == '/'
		      && (href_base)[strlen (href_base) - 2] == '/')
		    (href_base)[strlen (href_base) - 1] = '\0';
		}
	      break;

	    default:
	      tmp = uc_html_parser_add_base (href_base, tag->value);
	      tag->value = uc_utils_to_utf8 (tmp);
	      uc_utils_strstrip (tag->value);
	      g_free (tmp), tmp = NULL;
	    }

	  if (tag->type != LINK_TYPE_BASE_HREF)
	    tags = g_list_append (tags, tag);
	  else
	    {
	      g_free (tag->label), tag->label = NULL;
	      g_free (tag->value), tag->value = NULL;

	      g_free (tag), tag = NULL;
	    }
	}

      if (cur->children)
	tags = uc_html_parser_parse (tags, cur->children);

      cur = cur->next;
    }

  return tags;
}

/**
 * uc_html_parser_add_base:
 * @base: HTML base tag to add.
 * @url: URL for base tag concatenation.
 * 
 * Add the base tag value to the link if necessary.
 *
 * Returns: A new allocated string with new URL.
 */
static gchar *
uc_html_parser_add_base (const gchar * base, const gchar * url)
{
  gchar *path = NULL;
  gchar *file = NULL;
  gchar *new = NULL;
  gboolean is_args = FALSE;

  path = g_strdup (base);

  if (path == NULL || url == NULL || strstr (url, "://") != NULL ||
      (file = strrchr (path, '/')) == NULL || url[0] == '/')
    {
      g_free (path), path = NULL;
      return g_strdup (url);
    }

  if (*(file - 1) == '/')
    file = NULL;
  else
    {
      *file = '\0';
      file++;
    }

  is_args = (strchr ("?#", url[0]) != NULL);

  if (path[strlen (path) - 1] == '/')
    path[strlen (path) - 1] = '\0';

  new = (is_args == FALSE) ?
    g_strconcat (path, ((url[0] != '/') ? "/" : ""), url, NULL) :
    g_strconcat (path, "/", file, url, NULL);

  g_free (path), path = NULL;

  return new;
}

/**
 * uc_html_parser_node_new:
 * 
 * Create a new tag list node.
 *
 * Returns: A new allocated #UCHTMLTag node.
 */
UCHTMLTag *
uc_html_parser_node_new (void)
{
  UCHTMLTag *item = NULL;

  item = g_new0 (UCHTMLTag, 1);
  item->depth = 0;
  item->label = NULL;
  item->value = NULL;
  item->type = LINK_TYPE_NONE;

  return item;
}
