/*
 * Copyright (C) 2002, 2003, 2004 Emmanuel Saracco <esaracco@noos.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "check.h"
#include "connection.h"
#include "project.h"
#include "utils.h"

/**
 * uc_utils_get_meta_refresh_location:
 * @prop: A #UCLinkProperties node.
 * @tag: A #UCHTMLTag tag.
 *
 * If the page contain a refresh meta tag, replace the current page URL
 * with it.
 * 
 * Returns: TRUE if there was a REFRESH META tag.
 */
gboolean
uc_utils_get_meta_refresh_location (UCLinkProperties * prop, UCHTMLTag * tag)
{
  GList *item = NULL;
  gchar **tmp = NULL;
  gboolean ok = TRUE;
  gboolean ret = FALSE;

  if (prop == NULL || prop->metas == NULL)
    return FALSE;

  item = g_list_first (prop->metas);
  while (item != NULL && ok == TRUE)
    {
      UCHTMLTag *meta = (UCHTMLTag *) item->data;

      if (g_ascii_strcasecmp (meta->label, "refresh") == 0
	  && meta->value[0] == '0')
	{
	  tmp = g_strsplit (meta->value, ";", -1);

	  if (tmp[1] != NULL)
	    {
	      gchar *p = NULL;

	      p = tmp[1];
	      for (; p && *p != '='; p++);

	      if (p && ++p)
		{
		  g_free (tag->value), tag->value = NULL;
		  tag->value = g_strdup (p);

		  ret = TRUE;
		}

	      ok = FALSE;
	    }

	  g_strfreev (tmp), tmp = NULL;
	}

      item = g_list_next (item);
    }

  return ret;
}

/**
 * uc_utils_html_decode:
 * @str: The string to HTML decode.
 *
 * HTML decode (very basic stuff) a given string.
 * 
 * Returns: A new allocated HTML decoded string.
 */
gchar *
uc_utils_html_decode (const gchar * str)
{
  GString *new_str = NULL;
  gchar *ret = NULL;

  new_str = g_string_new ("");

  while (*str)
    {
      if (*str == '\n' || *str == '\r')
	;
      else if (*str == '&' && *(str + 1) == '#')
	{
	  gchar buf[4] = { 0 };
	  guint i = 0;

	  str += 2;

	  while (*str && g_ascii_isdigit (*str) && i < 4)
	    buf[i++] = *str++;
	  str--;

	  g_string_append_c (new_str, atoi (buf));
	}
      else if (*str == '%' && *(str + 2) != '\n')
	{
	  gchar buf[3] = { 0 };
	  guint i = 0;

	  buf[0] = *(++str);
	  buf[1] = *(++str);

	  sscanf (buf, "%x", &i);
	  g_string_append_c (new_str, i);
	}
      else
	new_str = g_string_append_c (new_str, *str);

      str++;
    }

  ret = new_str->str;
  g_string_free (new_str, FALSE), new_str = NULL;

  return ret;
}

/**
 * uc_utils_to_utf8:
 * @data: String to proceed.
 *
 * UTF-8 encode a given string.
 * Adapted from screem "screem_support_charset_convert ()"
 * function.
 * 
 * Returns: A new allocated UTF-8 encoded string.
  */
gchar *
uc_utils_to_utf8 (gchar * data)
{
  const gchar *end;
  gchar *ret = NULL;
  gsize len = 0;

  if (data != NULL)
    {
      if (g_utf8_validate (data, strlen (data), &end) == FALSE)
	{
	  const gchar *charset;
	  const char *encodings_to_try[2] = { 0, 0 };
	  gboolean utf8 = FALSE;

	  utf8 = g_get_charset (&charset);

	  if (utf8 == FALSE)
	    encodings_to_try[0] = charset;

	  if (g_ascii_strcasecmp (charset, "ISO-8859-1"))
	    encodings_to_try[1] = "ISO-8859-1";

	  if (encodings_to_try[0])
	    ret = g_convert (data, strlen (data), "UTF-8",
			     encodings_to_try[0], NULL, &len, NULL);

	  if (ret == NULL && encodings_to_try[1])
	    ret = g_convert (data, strlen (data), "UTF-8",
			     encodings_to_try[1], NULL, &len, NULL);
	}
      else
	ret = g_strdup (data);
    }

  if (ret == NULL)
    ret = g_strdup ("");

  return ret;
}

/**
 * uc_utils_get_yesno:
 * @yesno: A string containing "yes" or "no".
 *
 * Proceed a given string to see if it contains "yes" or "no".
 * 
 * Returns: TRUE if the given string is "yes".
 */
gboolean
uc_utils_get_yesno (const gchar * yesno)
{
  return (gboolean) (g_ascii_strcasecmp (yesno, "yes") == 0);
}

/**
 * uc_utils_debug:
 * @format: The message format.
 * @...: The parameters to insert into the format string.
 * 
 * Display debug informations.
 */
void
uc_utils_debug (const gchar * format, ...)
{
  va_list ap;
  gchar *s = NULL;

  if (uc_project_get_debug_mode () == FALSE)
    return;

  s = (gchar *) g_malloc0 (2048 + 1);

  va_start (ap, format);
  vsnprintf (s, 2048, format, ap);
  va_end (ap);

  g_print ("DEBUG: ");
  g_print (s);

  g_free (s), s = NULL;
}

/**
 * uc_utils_debug_dump_link_properties:
 * @prop: A #UCLinkProperties node.
 * 
 * Dump the link properties structure to the standard
 * output.
 */
void
uc_utils_debug_dump_link_properties (const UCLinkProperties * prop)
{
  if (uc_project_get_dump_properties () == FALSE)
    return;

  g_print ("\n\n"
	   "        link_type: %u\n"
	   "        link_value: %s\n"
	   "        url: %s\n"
	   "        label: %s\n"
	   "        protocol: %s\n"
	   "        h_name: %s\n"
	   "        port: %s\n"
	   "        path: %s\n"
	   "        args: %s\n"
	   "        domain: %s\n"
	   "        header_size: %u\n"
	   "        depth_level: %u\n"
	   "        is_parsable: %u\n"
	   "        is_downloadable: %u\n"
	   "        checked: %u\n",
	   prop->link_type,
	   prop->link_value,
	   prop->url,
	   prop->label,
	   prop->protocol,
	   prop->h_name,
	   prop->port,
	   prop->path,
	   prop->args,
	   prop->domain,
	   prop->header_size,
	   prop->depth_level, prop->is_parsable, prop->is_downloadable,
	   prop->checked);
}

/**
 * uc_utils_set_userfriendly_treeview_column:
 * @tv: Tree view to work with.
 * @position: Position of the column to work with.
 * 
 * Set a treeview column sortable and resizable.
 */
void
uc_utils_set_userfriendly_treeview_column (const GtkTreeView * tv,
					   const gint32 position)
{
  GtkTreeViewColumn *column = NULL;

  column = gtk_tree_view_get_column (GTK_TREE_VIEW (tv), position);
  if (column == NULL)
    return;

  gtk_tree_view_column_set_sort_column_id (column, position);
  gtk_tree_view_column_set_resizable (column, TRUE);
}

/**
 * uc_utils_get_file_content:
 * @path: Path of the file.
 * @length: Pointer to fill with file length.
 * 
 * Open and read a file to return its content.
 * 
 * Returns: A new allocated string with the content of a the file. 
 *          The @length argument contain the file length.
 */
gchar *
uc_utils_get_file_content (const gchar * path, gsize * length)
{
  gchar *buffer = NULL;
  GError *error = NULL;

  g_file_get_contents (path, &buffer, &(*length), &error);

  return buffer;
}

/* FIXME */
/**
 * uc_utils_email_is_valid:
 * @email: Email to check.
 * @check_mx: TRUE if we also must check the MX.
 * 
 * Check email syntax and MX if required.
 *
 * Returns: The #UCEmailStatus status of the email.
 *          Return codes are:
 *          - 0: bad syntax (UC_EMAIL_SYNTAX_BAD)
 *          - 1: syntax ok but MX not respond (UC_EMAIL_SYNTAX_MX_BAD)
 *          - 2: syntax ok and MX ok (UC_EMAIL_OK)
 */
UCEmailStatus
uc_utils_email_is_valid (const gchar * email, const gboolean check_mx)
{
  gchar *tmp = NULL;
  gchar *user = NULL;
  gchar *domain = NULL;
  gchar *email_tmp = NULL;
  gchar *mx = NULL;
  guint ret = UC_EMAIL_SYNTAX_BAD;

  uc_utils_split_email (email, &user, &domain);
  if (strlen (user) == 0 || strlen (domain) == 0)
    {
      g_free (user), user = NULL;
      g_free (domain), domain = NULL;

      return UC_EMAIL_SYNTAX_BAD;
    }

  email_tmp = g_strconcat (user, "@", domain, NULL);

  /* dirty syntax control :-( */
  if ((email_tmp == NULL) ||
      (strlen (email_tmp) == 0) ||
      (strchr ("@.", email_tmp[0])) ||
      (strchr ("@.", email_tmp[strlen (email_tmp) - 1])) ||
      (strchr (email_tmp, ' ')) || ((tmp = strchr (email_tmp, '@')) == NULL)
      || ((tmp = strchr (email_tmp, '.'))) == NULL)
    {
      g_free (email_tmp), email_tmp = NULL;

      return UC_EMAIL_SYNTAX_BAD;
    }

  /* check if MX is alive and listening on
   * port 25 */
  if (check_mx)
    {
      mx = uc_utils_get_mx (domain);

      ret = (uc_utils_mx_is_valid (mx)) ? UC_EMAIL_OK : UC_EMAIL_SYNTAX_MX_BAD;

      g_free (mx), mx = NULL;
    }
  else
    ret = UC_EMAIL_OK;

  g_free (user), user = NULL;
  g_free (domain), domain = NULL;

  return ret;
}

/**
 * uc_utils_split_email:
 * @email: Email to work with.
 * @user: Pointer for the "user" part of the email.
 * @domain: Pointer for the "domain" part of the email.
 * 
 * Fill the two parts of a email address. @user and @domain argument are
 * filled with new allocated strings.
 */
void
uc_utils_split_email (const gchar * email, gchar ** user, gchar ** domain)
{
  gchar **components = NULL;
  gchar *sep[] = {
    "@",
    "AT"
  };
  guint i = 0;

  if (email == NULL ||
      (strchr (email, '@') == NULL && strstr (email, "AT") == NULL))
    {
      *user = g_strdup ("");
      *domain = g_strdup ("");
    }
  else
    {
      i = (strchr (email, '@')) ? 0 : 1;

      components = g_strsplit (email, sep[i], 0);
      *user = g_strdup ((components[0]) ? components[0] : "");
      *domain = g_strdup ((components[1]) ? components[1] : "");

      g_strfreev (components), components = NULL;
    }
}

/**
 * uc_utils_get_mx:
 * @domain: Domain from which returning the corresponding MX.
 * 
 * Return the prefered MX for a given domain.
 * Function adapted from 
 * http://www.sslug.dk/emailarkiv/cprog/1999_10/msg00056.html
 *
 * Returns: A new allocated string with the corresponding MX.
 */
gchar *
uc_utils_get_mx (const gchar * domain)
{
  struct rrecord
  {
    gint16 r_type;
    gint16 r_class;
    guint32 r_ttl;
    gint16 r_length;
  };

  static gchar out[1000];
  guchar query[1000];
  HEADER *h;
  guchar *p;
  gint32 qcount, acount;
  gchar buf[1000];
  gint32 best;
  struct rrecord *r;

  /* ask libc to do the query:
   * look up 'domain'
   * class C_IN (internet)
   * type  T_MX (mail exchanger record)
   * record results in query, max len 1000
   */
  if (res_search (domain, C_IN, T_MX, query, (int) 1000) < 0)
    return g_strdup (domain);

  h = (HEADER *) query;

  /* skip over queries in packet */
  p = query + sizeof (HEADER);
  qcount = ntohs (h->qdcount);	/* # of queries present */
  acount = ntohs (h->ancount);	/* # of answers present */
  /* get past queries */
  while (qcount--)
    {
      p += dn_expand (query, query + 1000, p, buf, 1000);
      p += 4;			/* qclass, qtype */
    }

  /* now we are in the answers section. Just get the best exchanger,
   * we don't want to spend too much time messing around
   */
  best = 99999;
  out[0] = 0;
  while (acount--)
    {
      /* name that this answer refers to */
      p += dn_expand (query, query + 1000, p, buf, 1000);

      r = (struct rrecord *) p;
      if (htons (r->r_type) == T_MX)
	{			/* MX record */
	  gint32 pref;
	  guchar *p1 = p + 10;

	  pref = ntohs (*(gint16 *) p1);	/* preference */
	  p1 += 2;
	  if (pref < best)
	    {
	      /* get exchanger name */
	      dn_expand (query, query + 1000, p1, out, 1000);
	      best = pref;
	    }
	}
      p = p + 10 + ntohs (r->r_length);	/* skip RR header and data */
    }

  return (out[0]) ? g_strdup (out) : g_strdup (domain);
}

/**
 * uc_utils_mx_is_valid:
 * @mx: The MX to check.
 * 
 * Check if a given MX is valid or not.
 * 
 * Returns: TRUE if the given MX is alive and listen on port 25.
 */
gboolean
uc_utils_mx_is_valid (const gchar * mx)
{
  GTcpSocket *socket = NULL;
  gboolean ret = FALSE;

  socket = gnet_tcp_socket_connect (mx, 25);

  if (socket != NULL)
    {
      ret = TRUE;
      gnet_tcp_socket_delete (socket), socket = NULL;
    }

  return ret;
}

/**
 * uc_utils_get_ip:
 * @host: Host to work with.
 * 
 * Return the IP of the given host. if arg is already an IP,
 * return it as is.
 *
 * Returns: A new allocated string with the IP.
 */
gchar *
uc_utils_get_ip (const gchar * host)
{
  struct hostent *serverHostEnt = NULL;
  gchar *buffer = NULL;

  if ((serverHostEnt = gethostbyname (host)) != NULL)
    buffer = g_strdup (inet_ntoa
		       (*((struct in_addr *) serverHostEnt->h_addr_list[0])));
  else if ((in_addr_t) (inet_addr (host)) != (in_addr_t) - 1)
    buffer = g_strdup (host);
  else
    buffer = g_strdup ("");

  return buffer;
}

/**
 * uc_utils_string_format4display:
 * @label: Label to format.
 * @size: Size limit.
 *
 * format a given string to be displayed in UI.
 *
 * See: uc_utils_string_cut()
 * 
 * Returns: A new allocated string with the new formated label.
 */
gchar *
uc_utils_string_format4display (const gchar * label, const gsize size)
{
  gchar *ret = NULL;
  gchar *tmp = NULL;

  tmp = g_strdup (label);

  if (tmp != NULL)
    uc_utils_replacelr (uc_utils_strstrip (tmp), ' ');

  ret = uc_utils_string_cut (tmp, size);

  g_free (tmp), tmp = NULL;

  return ret;
}

/**
 * uc_utils_string_cut:
 * @label: Label to cut.
 * @size: Size limit.
 *
 * Cut a given string to fit with the given length.
 * 
 * See: uc_utils_string_format4display()
 * 
 * Returns: A new allocated string with the cutted label.
 */
gchar *
uc_utils_string_cut (const gchar * label, const gsize size)
{
  gchar *ret = NULL;
  gchar *tmp = NULL;

  tmp = g_strdup (label);

  if (tmp != NULL && size > 0 && strlen (tmp) > size - 3)
    {
      gchar *tmp1 = NULL;

      tmp1 = g_malloc0 (sizeof (gchar) * size + 3 + 1);
      memcpy (tmp1, tmp, size);
      strncat (tmp1, "...", 3);

      g_free (tmp), tmp = NULL;
      tmp = tmp1;
    }

  ret = uc_utils_to_utf8 (tmp);

  g_free (tmp), tmp = NULL;

  return ret;
}

/**
 * uc_utils_replace:
 * @str: String to work with.
 * @old: String to replace.
 * @new: New string to replace old with.
 * 
 * Replace one or more characters in a given string.
 *
 * See: uc_utils_replace1()
 * 
 * Returns: A new allocated string.
 */
gchar *
uc_utils_replace (const gchar * str, const gchar * old, const gchar * new)
{
  guint32 i = 0;
  guint32 str_len = strlen (str);
  guint32 old_len = strlen (old);
  GString *tmp = g_string_new ("");
  gchar *ret = NULL;

  for (i = 0; i < str_len; i++)
    if (memcmp (&str[i], old, old_len) == 0)
      tmp = g_string_append (tmp, new), i += old_len - 1;
    else
      tmp = g_string_append_c (tmp, str[i]);

  ret = tmp->str;
  g_string_free (tmp, FALSE), tmp = NULL;

  return ret;
}

/**
 * uc_utils_replace1:
 * @string: String to work with.
 * @c1: Character to replace.
 * @c2: Replacement.
 * 
 * Replace a caracter by another (but do not allocate new memory for the 
 * result string).
 *
 * See: uc_utils_replace()
 * 
 * Returns: The modified string.
 */
gchar *
uc_utils_replace1 (gchar * string, const gchar c1, const char c2)
{
  gchar *p = string;

  do
    {
      if (*p == c1)
	*p = c2;
    }
  while (*p++);

  return string;
}

/**
 * uc_utils_replacelr:
 * @string: String to work with.
 * @c: Character for LR replacement.
 * 
 * Remove all line return in a given string.
 *
 * Returns: The modified string.
 */
gchar *
uc_utils_replacelr (gchar * string, const gchar c)
{
  gchar *p = string;

  do
    {
      if (*p == '\n' || *p == '\r')
	*p = c;
    }
  while (*p++);

  return string;
}

/**
 * uc_utils_strchomp:
 * @string: A string.
 * 
 * Remove all spaces or separators before a
 * given string.
 *
 * Returns: The modified string.
 */
gchar *
uc_utils_strchomp (gchar * string)
{
  gsize len = 0;

  len = strlen (string);
  while (len--)
    if ((string[len] == '\n') ||
	(string[len] == '\r') ||
	(string[len] == '\t') || (g_ascii_isspace (string[len])))
      string[len] = '\0';
    else
      break;

  return string;
}

/**
 * uc_utils_strchug:
 * @string: A string.
 * 
 * Remove all spaces or separator after a
 * given string.
 *
 * Returns: The modified string.
 */
gchar *
uc_utils_strchug (gchar * string)
{
  const gchar *start;

  for (start = (gchar *) string;
       (*start) && ((*start == '\n') ||
		    (*start == '\r') ||
		    (*start == '\t') || (g_ascii_isspace (*start))); start++);

  g_memmove (string, start, strlen ((gchar *) start) + 1);

  return string;
}

/**
 * uc_utils_convert_uid2file:
 * @uid: The uid of a #UCLinkProperties node.
 * 
 * Convert a cache uid to filename.
 *
 * Returns: The corresponding path. A new allocated string.
 */
gchar *
uc_utils_convert_uid2file (const guint32 uid)
{
  return g_strdup_printf ("%s/%s/%d",
			  uc_project_get_working_path (),
			  uc_project_get_cache_name (), uid);;
}

/**
 * uc_utils_get_gnome_browser_conf:
 * 
 * Get the current default gnome web browser or "mozilla" if it
 * does not find it.
 * 
 * Returns: The current gnome default web browser
 */
gchar *
uc_utils_get_gnome_browser_conf (void)
{
  GConfClient *client = NULL;
  gchar *ret = NULL;

  client = gconf_client_get_default ();

  ret =
    gconf_client_get_string (client,
			     "/desktop/gnome/url-handlers/http/command", NULL);

  g_object_unref (client), client = NULL;

  return (ret == NULL) ? g_strdup ("mozilla") : ret;
}

/**
 * uc_utils_get_gnome_proxy_conf:
 * @host: A string to fill for the host.
 * @port: A integer to fill for the port.
 * 
 * Fill given args with the gconf values for the gnome network proxy 
 * settings.
 */
void
uc_utils_get_gnome_proxy_conf (gchar ** host, guint * port)
{
  GConfClient *client = NULL;

  client = gconf_client_get_default ();

  if (gconf_client_get_bool (client, "/system/http_proxy/use_http_proxy", NULL))
    {
      *host = gconf_client_get_string (client, "/system/http_proxy/host", NULL);
      *port = gconf_client_get_int (client, "/system/http_proxy/port", NULL);

      if (*port == 0)
	*port = 8080;
    }

  if (*host == NULL)
    *host = g_strdup ("");

  g_object_unref (client), client = NULL;
}

/**
 * uc_utils_disk_free_enough:
 * @requested_size: The size needed on the partition.
 * 
 * Check if there is enought space on the partition to write on it.
 * 
 * Returns: %TRUE if it exist enough space on the parition
 *          of the given path to write on it.
 */
gboolean
uc_utils_disk_free_enough (const guint32 requested_size)
{
  struct statfs sfs;
  guint32 requested_blocks = 0;
  guint32 secure_add = UC_SECURE_BYTES_ADDING;

  statfs (uc_project_get_working_path (), &sfs);

  secure_add /= sfs.f_bsize;
  requested_blocks = ((requested_size / sfs.f_bsize) <= 0) ?
    1 : requested_size / sfs.f_bsize;

  return ((requested_blocks + secure_add) < sfs.f_bavail);
}

/**
 * uc_utils_clean_tag_link_value:
 * @value: String to clean.
 * 
 * Clean a link value string.
 *
 * Returns: A cleaned value. A new allocated string.
 */
gchar *
uc_utils_clean_tag_link_value (gchar * value)
{
  GString *buffer = NULL;
  gchar *ret = NULL;
  const gchar *p = value;

  g_return_val_if_fail (value != NULL, g_strdup (""));

  buffer = g_string_new ("");
  while (*p)
    {
      if (*p != '\'' && *p != '"' && *p != '>')
	g_string_append_c (buffer, *p);
      p++;
    }

  ret = buffer->str;
  g_string_free (buffer, FALSE), buffer = NULL;

  return ret;
}

/**
 * uc_utils_search_string_next:
 * @buf: A #gpointer to the source buffer.
 * @str: The string to search.
 * @limit_char: The char to stop if found.
 * 
 * Search for the next char after the searched string.
 *
 * Returns: If found, retrun a pointer on the next char,
 *          else return the original pointer.
 */
gpointer
uc_utils_search_string_next (gpointer buf, const gchar * str,
			     const gchar limit_char)
{
  gpointer sav = buf;
  gchar *buffer = (gchar *) buf;

  if (str == NULL || strlen (str) == 0 || buffer == NULL
      || strlen (buffer) < strlen (str))
    return (gpointer) buf;

  while (*buffer != '\0' &&
	 uc_utils_memcasecmp (buffer, str) == FALSE && *buffer != limit_char)
    buffer++;

  if (*buffer != limit_char)
    sav = (buffer += strlen (str));

  return sav;
}

/**
 * uc_utils_memcasecmp:
 * @str1: First string to compare.
 * @str2: Second string to compare.
 * 
 * Comparison between two strings, based on the length of
 * the second string.
 *
 * Returns: %TRUE if the beginning of %str1 match with %str2.
 */
gboolean
uc_utils_memcasecmp (const gchar * str1, const gchar * str2)
{
  gchar *tmp = NULL;
  guint32 len = 0;
  gboolean ret = FALSE;

  if (str1 == NULL || str2 == NULL)
    return FALSE;

  len = strlen (str2);

  if (strlen (str1) < len)
    return FALSE;

  if (memcmp (str1, str2, len) == 0)
    return TRUE;

  tmp = g_malloc0 (len + 1), memcpy (tmp, str1, len);
  ret = (g_ascii_strcasecmp (tmp, str2) == 0);
  g_free (tmp), tmp = NULL;

  return ret;
}

/**
 * uc_utils_mkdirs:
 * @path: Path to create.
 * @create_all: %TRUE if we must create intermediar paths (mkdir -p).
 * 
 * Make given path -- create intermediate paths if they don't exist -- 
 * and test if we are authorized to write in.
 *
 * Returns: %TRUE if all is ok.
 */
gboolean
uc_utils_mkdirs (const gchar * path, const gboolean create_all)
{
  gchar test_path[UC_BUFFER_LEN + 1] = { 0 };
  gchar **paths = NULL;
  gboolean ret = FALSE;
  gint32 fd;
  gint32 i = 0;

  if (create_all)
    {
      paths = g_strsplit (path, "/", 255);
      while (paths[i] != NULL)
	{
	  if (strlen (paths[i]) > 0)
	    {
	      strncat (test_path, "/", UC_BUFFER_LEN - strlen (test_path));
	      if (strncat
		  (test_path, paths[i],
		   UC_BUFFER_LEN - strlen (test_path)) == NULL)
		{
		  g_strfreev (paths), paths = NULL;
		  return FALSE;
		}

	      mkdir (test_path, S_IRWXU);
	    }
	  i++;
	}
    }
  else
    {
      mkdir (path, S_IRWXU);
      strncat (test_path, path, UC_BUFFER_LEN - strlen (test_path));
    }

  strncat (test_path, "/filetest", UC_BUFFER_LEN - strlen (test_path));
  fd = open (test_path, O_CREAT, S_IRUSR | S_IWUSR);
  if (fd >= 0)
    {
      close (fd);
      unlink (test_path);
      ret = TRUE;
    }

  g_strfreev (paths), paths = NULL;
  return ret;
}

/**
 * uc_utils_rmfiles:
 * @path: The path in witch deleting files.
 * 
 * Remove all files in a given directory.
 */
void
uc_utils_rmfiles (const gchar * path)
{
  DIR *dir = NULL;
  struct dirent *file = NULL;

  dir = opendir (path);
  if (dir == NULL)
    return;

  while ((file = readdir (dir)) != NULL)
    {
      gchar *file_path = g_strconcat (path, "/", file->d_name,
				      NULL);
      unlink (file_path);
      g_free (file_path), file_path = NULL;
    }
  closedir (dir);
}

/**
 * uc_utils_copy:
 * @src: Source file.
 * @dest: Destination file.
 *
 * Copy a file from %src to %dest.
 */
void
uc_utils_copy (const gchar * src, const gchar * dest)
{
  gchar *buffer = NULL;
  gsize length = 0;
  FILE *fd = NULL;

  g_assert (g_file_test (src, G_FILE_TEST_EXISTS) == TRUE);

  g_file_get_contents (src, &buffer, &length, NULL);
  fd = fopen (dest, "w");
  fwrite (buffer, 1, length, fd);
  fclose (fd);

  g_free (buffer), buffer = NULL;
}

/**
 * uc_utils_copy_files:
 * @src: Source path.
 * @dest: Destination path.
 * 
 * Copy all the file from a given directorie in another.
 */
void
uc_utils_copy_files (const gchar * src, const gchar * dest)
{
  DIR *dir = NULL;
  struct dirent *file = NULL;
  FILE *fd = NULL;
  gchar *file_src = NULL;
  gchar *file_dest = NULL;

  dir = opendir (src);
  if (dir == NULL)
    return;

  while ((file = readdir (dir)))
    {
      gchar *buffer = NULL;
      gsize length = 0;

      if (strcmp (file->d_name, ".") == 0 || strcmp (file->d_name, "..") == 0)
	continue;

      file_src = g_strconcat (src, "/", file->d_name, NULL);
      file_dest = g_strconcat (dest, "/", file->d_name, NULL);

      g_file_get_contents (file_src, &buffer, &length, NULL);
      g_assert (buffer != NULL);
      fd = fopen (file_dest, "w");
      g_assert (fd != NULL);
      fwrite (buffer, 1, length, fd);
      fclose (fd);

      g_free (file_src), file_src = NULL;
      g_free (file_dest), file_dest = NULL;
      g_free (buffer), buffer = NULL;
    }

  closedir (dir);
}

/**
 * uc_utils_rmdirs:
 * @path: The path to remove.
 * @delete_all: If %TRUE, remove all intermediar paths.
 * 
 * Remove a given path -- remove intermediate paths too.
 */
void
uc_utils_rmdirs (const gchar * path, const gboolean delete_all)
{
  gchar **paths = NULL;
  gint32 i = 0;

  if (delete_all == TRUE)
    {
      paths = g_strsplit (path, "/", 255);
      i = uc_utils_vector_length (paths);
      while (i >= 0)
	{
	  gchar dir[UC_BUFFER_LEN + 1] = { 0 };
	  guint j = 0;

	  for (j = 0; j < i; j++)
	    {
	      if (strlen (paths[j]) > 0)
		{
		  if (strncat (dir, "/",
			       UC_BUFFER_LEN - strlen (dir)) == NULL ||
		      strncat (dir, paths[j],
			       UC_BUFFER_LEN - strlen (dir)) == NULL)
		    {
		      g_strfreev (paths), paths = NULL;
		      return;
		    }
		}
	    }

	  uc_utils_rmfiles (dir);
	  rmdir (dir);

	  i--;
	}

      g_strfreev (paths), paths = NULL;
    }
  else
    {
      uc_utils_rmfiles (path);
      rmdir (path);
    }
}

/**
 * uc_utils_vector_length:
 * @data: A vector.
 * 
 * Calculate the length of the given vector.
 * 
 * Returns: The length of a given vector.
 */
guint
uc_utils_vector_length (gpointer data)
{
  guint32 i = 0;
  gchar **vector = (gchar **) data;

  while (vector[i++]);

  return --i;
}

/**
 * uc_utils_test_socket_open:
 * @sock: The socket to test.
 * 
 * Check if a connection if alive or not.
 * -> taken from wget source code
 *
 *  Returns: %TRUE if it is alive.
 */
gboolean
uc_utils_test_socket_open (const guint sock)
{
  fd_set check_set;
  struct timeval to;
  gboolean ret = FALSE;

  /* Check if we still have a valid (non-EOF) connection.  From Andrew
   * Maholski's code in the Unix Socket FAQ.  */

  FD_ZERO (&check_set);
  FD_SET (sock, &check_set);

  /* Wait one microsecond */
  to.tv_sec = 0;
  to.tv_usec = 1;

  /* If we get a timeout, then that means still connected */
  if (select (sock + 1, &check_set, NULL, NULL, &to) == 0)
    {
      /* Connection is valid (not EOF), so continue */
      ret = TRUE;
    }

  return ret;
}

/**
 * uc_utils_strdup_delim:
 * @begin: The source position.
 * @end: The end position.
 * 
 * Make a string with 2 given pointers.
 *
 * Returns: A new allocated string.
 */
gchar *
uc_utils_strdup_delim (const gchar * begin, const gchar * end)
{
  gchar *res = NULL;

  res = (gchar *) g_malloc (end - begin + 1);
  memcpy (res, begin, end - begin);
  res[end - begin] = '\0';

  return res;
}

/**
 * uc_utils_strpbrk_or_eos:
 * @str: A string.
 * @accept: A string with characters to search for.
 * 
 * Return a pointer on a given char address or on the end of 
 * the passed string.
 *
 * Returns: A pointer on the new offset.
 */
gchar *
uc_utils_strpbrk_or_eos (const gchar * str, const gchar * accept)
{
  gchar *p = NULL;

  p = strpbrk (str, accept);
  if (p == NULL)
    p = (gchar *) str + strlen (str);

  return p;
}

/**
 * uc_utils_array_is_in:
 * @item: Item to search for.
 * @array: Vector to search in.
 * @size: Size of the vector.
 * 
 * Search for the first argument in the second argument vector.
 *
 * Returns: %TRUE if the first argument is in the second argument vector.
 */
gboolean
uc_utils_array_is_in (const gchar * item, gpointer array, const guint size)
{
  gchar **items = (gchar **) array;
  guint i = 0;

  for (i = 0; i < size; i++)
    if (strcmp (item, items[i]) == 0)
      return TRUE;

  return FALSE;
}

/**
 * uc_utils_get_server_from_header_field:
 * @field: A string from the "Server" HTTP header field.
 *
 * Extract server name/type from a HTTP header "Server" field.
 *
 * Returns: A string with the HTTP server. A new allocated string.
 */
gchar *
uc_utils_get_server_from_header_field (gchar * field)
{
  const gchar *b = NULL;
  const gchar *e = NULL;

  b = field;
  e = strchr (b, ' ');

  if (e == NULL)
    return g_strdup (b);

  if (*(e + 1) == '(')
    e++, e = strchr (e, ')');

  return uc_utils_strdup_delim (b, e + 1);
}

gchar *
uc_utils_get_string_from_size (const gsize size)
{
  gchar *ret = NULL;

  if (size < 1024)
    ret = g_strdup_printf (_("%u bytes"), size);
  else if (size < (1024 * 1024))
    ret = g_strdup_printf (_("%.2f KB (%u bytes)"),
			   (gfloat) (size / 1024.0), size);
  else
    ret = g_strdup_printf (_("%.2f MB (%u bytes)"),
			   (gfloat) (size / (1024.0 * 1024.0)), size);

  return ret;
}

gboolean
uc_utils_ftp_code_search (const gchar * buffer, const gchar * code,
			  const gint len)
{
  if (len < strlen (code))
    return FALSE;

  return (memcmp (buffer, code, strlen (code)) == 0);
}
